/* This file is part of the KDE project
   Copyright (c) 2004 Kevin Ottens <ervin ipsquad net>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#include "mediumbutton.h"
#include "mediumpopupmenu.h"

#include <qpainter.h>
#include <qdrawutil.h>
#include <qpopupmenu.h>
#include <qtooltip.h>

#include <kmessagebox.h>
#include <kmimetype.h>
#include <klocale.h>
#include <kdesktopfile.h>
#include <krun.h>
#include <kglobalsettings.h>
#include <kcursor.h>
#include <kapplication.h>
#include <kipc.h>
#include <kiconloader.h>
#include <kurldrag.h>

#include <konq_operations.h>


MediumButton::MediumButton(QWidget *parent, const KFileItem &fileItem)
	: QButton(parent), mFileItem(fileItem)
{
	setBackgroundMode(PaletteBackground);
	setBackgroundOrigin(AncestorOrigin);

	setMouseTracking(true);

	mHighlight = false;

	QToolTip::add(this, mFileItem.name());

	mChangeCursorOverItem = KGlobalSettings::changeCursorOverIcon();

        if(mChangeCursorOverItem)
        {
		setCursor(KCursor().handCursor());
        }
        else
        {
		unsetCursor();
        }

        refreshIcon();

	resize(20, 20);

	slotSettingsChanged(KApplication::SETTINGS_MOUSE);

	connect(kapp, SIGNAL(settingsChanged(int)), SLOT(slotSettingsChanged(int)));
	connect(kapp, SIGNAL(iconChanged(int)), SLOT(slotIconChanged(int)));

	kapp->addKipcEventMask(KIPC::SettingsChanged);
	kapp->addKipcEventMask(KIPC::IconChanged);

	setAcceptDrops(mFileItem.isWritable());
	connect(&mOpenTimer, SIGNAL(timeout()), SLOT(slotDragOpen()));
}

MediumButton::~MediumButton()
{
}

const KFileItem &MediumButton::fileItem() const
{
	return mFileItem;
}

void MediumButton::setFileItem(const KFileItem &fileItem)
{
	mFileItem = fileItem;
	setAcceptDrops(mFileItem.isWritable());
	QToolTip::remove(this);
	QToolTip::add(this, mFileItem.name());
	refreshIcon();
}


void MediumButton::refreshIcon()
{
	KMimeType::Ptr mime = mFileItem.determineMimeType();
	mIcon = mime->pixmap(KIcon::Panel, width()-4, KIcon::DefaultState);
	mIconHighlight = mime->pixmap(KIcon::Panel, width()-4, KIcon::ActiveState);
	repaint();
}


void MediumButton::slotSettingsChanged(int category)
{
	if(category != KApplication::SETTINGS_MOUSE) return;

	mChangeCursorOverItem = KGlobalSettings::changeCursorOverIcon();

        if(mChangeCursorOverItem)
        {
		setCursor(KCursor().handCursor());
        }
        else
        {
		unsetCursor();
        }
}

void MediumButton::slotIconChanged(int)
{
	refreshIcon();
}

void MediumButton::paintEvent(QPaintEvent*)
{
	QPainter p( this );
	drawButton( &p );
}

void MediumButton::drawButton(QPainter *p)
{
	if (parentWidget() && parentWidget()->backgroundPixmap())
		p->drawPixmap(0, 0, *(parentWidget()->backgroundPixmap()), x(), y());
	else
		p->fillRect(rect(), colorGroup().brush(QColorGroup::Background));

	if(isDown() || isOn())
	{
		p->fillRect(rect(), colorGroup().brush(QColorGroup::Mid));
		qDrawWinButton(p, 0, 0, width(), height(), colorGroup(), true);
	}

	drawButtonLabel(p);
}

void MediumButton::drawButtonLabel(QPainter *p)
{
	QPixmap *pix = &mIcon;
	if(mHighlight) pix = &mIconHighlight;

	int d = 2;
	if(isDown() || isOn()) d++;

	p->drawPixmap(d, d, *pix);
}

void MediumButton::enterEvent(QEvent* /*e*/)
{
	mHighlight = true;
	repaint();
}

void MediumButton::leaveEvent(QEvent* /*e*/)
{
	mHighlight = false;
	repaint();
}

void MediumButton::resizeEvent(QResizeEvent * /*e*/)
{
	refreshIcon();
}

void MediumButton::mousePressEvent(QMouseEvent *e)
{
	if(e->button()==LeftButton)
	{
		MediumPopupMenu menu(mFileItem, this);

		int local_x = 0;
		int local_y = 0;

		switch(mPanelPosition)
		{
		case KPanelApplet::pBottom:
			local_y-= menu.sizeHint().height();
			break;
		case KPanelApplet::pTop:
			local_y+= mIcon.height()+8;
			break;
		case KPanelApplet::pRight:
			local_x-= menu.sizeHint().width();
			break;
		case KPanelApplet::pLeft:
			local_x+= mIcon.width()+8;
			break;
		}

		setDown(true);
		setOn(true);

		repaint();

		menu.exec(this->mapToGlobal(QPoint(local_x, local_y)));

		setDown(false);
		setOn(false);

		repaint();
	}
	else
	{
		QButton::mousePressEvent(e);
	}
}

void MediumButton::dragEnterEvent(QDragEnterEvent* e)
{
	if (mFileItem.isWritable())
	{
		mOpenTimer.start(1000, true);
		e->accept(true);
	}
}

void MediumButton::dragLeaveEvent(QDragLeaveEvent* e)
{
	mOpenTimer.stop();

	QButton::dragLeaveEvent( e );
}

void MediumButton::dropEvent(QDropEvent *e)
{
	mOpenTimer.stop();

	KonqOperations::doDrop(&mFileItem, mFileItem.url(), e, this);
}

void MediumButton::slotDragOpen()
{
	mFileItem.run();
}


void MediumButton::setPanelPosition(KPanelApplet::Position position)
{
	mPanelPosition = position;
}

#include "mediumbutton.moc"
