/* This file is part of the KDE project
   Copyright (c) 2004 Kevin Ottens <ervin ipsquad net>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#include "mediumpopupmenu.h"

#include <qdir.h>
#include <klocale.h>
#include <kstandarddirs.h>
#include <kapplication.h>
#include <kmimetype.h>
#include <kiconloader.h>
#include <kglobalsettings.h>
#include <ktrader.h>
#include <kaction.h>
#include <qdatastream.h>
#include <dcopclient.h>
#include <dcopref.h>
#include <kmessagebox.h>

#define OPEN_SERVICE 0

MediumPopupMenu::MediumPopupMenu(const KFileItem &fileItem, QWidget *parent)
	: KPopupMenu(parent, "medium_popupmenu"), mFileItem(fileItem), mServicesSubMenu(0)
{
	insertTitle(mFileItem.name());

	queryMediaManager();
	loadServices();

	insertItem(i18n("Open"), OPEN_SERVICE);

	int id = OPEN_SERVICE+1;
	KIconLoader loader;

	if(!mMediumServices.isEmpty())
	{
		QValueList<KDEDesktopMimeType::Service>::Iterator it = mMediumServices.begin();
		QValueList<KDEDesktopMimeType::Service>::Iterator end = mMediumServices.end();

		for( ; it != end; ++it )
		{
			QIconSet icon = loader.loadIconSet((*it).m_strIcon, KIcon::Small, 0, true);
			insertItem(icon, (*it).m_strName, id);
			mServicesMap[id++]=*it;
		}
	}

	if(!mDirectoryServices.isEmpty())
	{
		KPopupMenu *menu;

		if(mMediumServices.isEmpty())
		{
			menu = this;
		}
		else
		{
			menu = &mServicesSubMenu;
			insertItem(i18n("Services"), menu);
		}

		QValueList<KDEDesktopMimeType::Service>::Iterator it = mDirectoryServices.begin();
		QValueList<KDEDesktopMimeType::Service>::Iterator end = mDirectoryServices.end();

		for( ; it != end; ++it )
		{
			QIconSet icon = loader.loadIconSet((*it).m_strIcon, KIcon::Small, 0, true);
			menu->insertItem(icon, (*it).m_strName, id);
			mServicesMap[id++]=*it;
		}
	}

	connect(this, SIGNAL( activated(int) ), this, SLOT( slotRunService(int) ));
	connect(&mServicesSubMenu, SIGNAL( activated(int) ), this, SLOT( slotRunService(int) ));
}

MediumPopupMenu::~MediumPopupMenu()
{

}

void MediumPopupMenu::queryMediaManager()
{
	QString name = mFileItem.url().fileName();
	DCOPRef mediamanager("kded", "mediamanager");
	DCOPReply reply = mediamanager.call( "properties", name );

	if ( reply.isValid() )
	{
		QStringList properties = reply;

		bool isMountable = properties[4]=="true";

		if ( isMountable )
		{
			mBaseURL = KURL(properties[6]);
		}
		else
		{
			mBaseURL = KURL(properties[9]);
		}
	}
}

void MediumPopupMenu::loadServices()
{
	QStringList dirs = KGlobal::dirs()->findDirs("data", "konqueror/servicemenus/");
	QStringList::ConstIterator dIt = dirs.begin();
	QStringList::ConstIterator dEnd = dirs.end();

	for(; dIt != dEnd; ++dIt)
	{
		QDir dir(*dIt);

		QStringList entries = dir.entryList(QDir::Files);
		QStringList::ConstIterator eIt = entries.begin();
		QStringList::ConstIterator eEnd = entries.end();

		for(; eIt != eEnd; ++eIt)
		{
			KSimpleConfig cfg(*dIt+*eIt, true);
			cfg.setDesktopGroup();

			if(cfg.hasKey("X-KDE-AuthorizeAction"))
			{
				bool ok = true;
				QStringList list = cfg.readListEntry("X-KDE-AuthorizeAction");

				if(kapp && !list.isEmpty())
				{
					for(QStringList::ConstIterator it = list.begin();
					    it != list.end();
					    ++it)
					{
						if (!kapp->authorize((*it).stripWhiteSpace()))
						{
							ok = false;
							break;
						}
					}
				}

				if(!ok) continue;
			}

			if(cfg.hasKey( "Actions" ) && cfg.hasKey( "ServiceTypes" ))
			{
				QStringList types = cfg.readListEntry("ServiceTypes");
				if( types.contains(mFileItem.mimetype()) )
				{
					mMediumServices += KDEDesktopMimeType::userDefinedServices(*dIt+*eIt, true);
				}
				else if( types.contains("inode/directory") )
				{
					mDirectoryServices += KDEDesktopMimeType::userDefinedServices(*dIt+*eIt, true);
				}
			}
		}
	}
}


bool operator==(const KDEDesktopMimeType::Service &a, const KDEDesktopMimeType::Service &b)
{
	return a.m_strName==b.m_strName
	    && a.m_strIcon==b.m_strIcon
	    && a.m_strExec==b.m_strExec
	    && a.m_type==b.m_type
	    && a.m_display==b.m_display;
}

void MediumPopupMenu::slotRunService(int id)
{
	if(id==OPEN_SERVICE)
	{
		mFileItem.run();
		return;
	}

	KDEDesktopMimeType::Service service = mServicesMap[id];
	KURL url = mBaseURL;

	if(mMediumServices.contains(service))
	{
		url = mFileItem.url();
	}

	KDEDesktopMimeType::executeService(url, service);
}

#include "mediumpopupmenu.moc"
